%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Programma per il calcolo della trasformata di Fourier di un
% segnale 'segnale(t,tau)' ad energia finita mediante DFT (FFT). 
% Il programma richiede in input le veriabili T,N e fmax.
% T: periodo di ripetizione di x(t)
% N: numero dei campioni
% fmax: massima frequenza da visualizzare
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear     % pulisce il workspace
close all % chiude tutte le figure

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
% LETTURA PARAMETRI 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
tau=input('Parametro dell''impulso [sec]= ');
T=input('Periodo di ripetizione nel tempo [sec]= ');
N=input('Numero di campioni [meglio se 2^k] = ');
fmax=input('Max freq. dello spettro visualizz. [Hz]= ');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PASSI CAMPIONAMENTO 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
dt=T/N;
df=1/T;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CAMPION. DELLA RIPETIZIONE PER. DI x(t) 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for i=1:N, 
    tempo(i)=(i-1)*dt;
    x(i)=segnale(tempo(i),tau);
    xr(i)=segnale(tempo(i),tau)+segnale(tempo(i)-T,tau)+segnale(tempo(i)-2*T,tau)+segnale(tempo(i)+T,tau)+segnale(tempo(i)+2*T,tau); 
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% GRAFICO x(t) e RIP. PERIODICA 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
subplot(321);plot(tempo,x),xlabel('time [s]'),ylabel('x(t)');
subplot(322);plot(tempo,xr),xlabel('time [s]'),ylabel('xr(t)');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CALCOLO SPETTRO MEDIANTE FFT 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
y=dt*fft(xr);
freq=[0:df:(N-1)*df];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% GRAFICO RIPETIZIONE PER. DELLO SPETTRO
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
subplot(323),plot(freq,abs(y)),xlabel('freq. [Hz]'),ylabel('|Xr(f)|');
subplot(324),plot(freq,angle(y)/pi*180),v=axis,
v(3)=-180,v(4)=180,axis(v),xlabel('freq. [Hz]'),ylabel('arg[Xr(f)] (degrees)');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PORZIONE DI SPETTRO da 0-fmax
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% resize dei vettori
npunti=min(round(fmax*T),N);
freq=freq(1:npunti);
y=y(1:npunti);

subplot(325),plot(freq,abs(y)),xlabel('freq. [Hz]'),ylabel('|X(f)|');
subplot(326),plot(freq,angle(y)/pi*180),
v=axis,v(3)=-180,v(4)=180,axis(v),xlabel('freq. [Hz]'),ylabel('arg[X(f)] (degrees)');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CONFRONTO CON SPETTRO TEORICO 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Xteo = j.*(2.*pi).^(5/4).*tau.^(1.5).*freq.*exp(-(pi.*freq.*tau).^2);

% STAMPA SPETTRO
figure  % mantiene il grafico precedente e ne crea uno nuovo
plot(freq,abs(y),'*',freq,abs(Xteo)),xlabel('freq. [Hz]'),ylabel('|X(f)|');


